# header: importing useful modules and functions

import csv
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import fmin

#%%#########################################################
# PART 2: Analysis of IPCC/CMIP5 Earth system model results
############################################################

#============
# SECTION 1.1
#============

# ancillary lists for loading data
model_list = ['ACCESS-10','ACCESS-13','BCC-CSM-11','BCC-CSM-11m','CanESM2','CCSM4','CNRM-CM5','CNRM-CM5-2','CSIRO-Mk360','GFDL-CM3','GFDL-ESM2G','GFDL-ESM2M','GISS-E2-H','GISS-E2-R','HadGEM2-ES','IPSL-CM5A-LR','IPSL-CM5A-MR','IPSL-CM5B-LR','MIROC5','MIROC-ESM','MPI-ESM-LR','MPI-ESM-MR','MPI-ESM-P','MRI-CGCM3','NorESM1-M']
model_length = {'ACCESS-10':150,'ACCESS-13':151,'BCC-CSM-11':150,'BCC-CSM-11m':150,'CanESM2':150,'CCSM4':151,'CNRM-CM5':150,'CNRM-CM5-2':140,'CSIRO-Mk360':150,'GFDL-CM3':150,'GFDL-ESM2G':300,'GFDL-ESM2M':300,'GISS-E2-H':151,'GISS-E2-R':151,'HadGEM2-ES':151,'IPSL-CM5A-LR':260,'IPSL-CM5A-MR':140,'IPSL-CM5B-LR':160,'MIROC5':151,'MIROC-ESM':150,'MPI-ESM-LR':150,'MPI-ESM-MR':150,'MPI-ESM-P':150,'MRI-CGCM3':150,'NorESM1-M':150}

# create array of final variables
# tas in {degC}; rsdt,rlut,rsut in {W/m2}
for var in ['tas','rsdt','rsut','rlut']:
#    exec(var+'_ctrl = np.zeros((np.max(model_length.values()),len(model_list))) * np.nan')
#    exec(var+'_quad = np.zeros((np.max(model_length.values()),len(model_list))) * np.nan')
    exec(var+'_ctrl = np.zeros((300,len(model_list))) * np.nan') #-> control simulation with preindustrial atmCO2 = 284ppm
    exec(var+'_quad = np.zeros((300,len(model_list))) * np.nan') #-> 4xCO2 simulation atmCO2=1136ppm

# load pre-processed CMIP5 results
for mod in model_list:
    for sim in ['ctrl','quad']:
        TMP = np.array([line for line in csv.reader(open('data/Climate_CMIP5/Climate_'+mod+'.'+sim+'_global.csv','r'))][1:],dtype=float)
        lgd = [line for line in csv.reader(open('data/Climate_CMIP5/Climate_'+mod+'.'+sim+'_global.csv','r'))][0]
        for var in ['tas','rsdt','rsut','rlut']:
            exec(var+'_'+sim+'[:len(TMP),model_list.index(mod)] = TMP[:,lgd.index(var)]')


# create net radiative budget variable
netrad_ctrl = rsdt_ctrl - rsut_ctrl - rlut_ctrl
netrad_quad = rsdt_quad - rsut_quad - rlut_quad

# Compute Radiative Imbalance and Warming
netImbalance = netrad_quad - netrad_ctrl
Warming      = tas_quad- tas_ctrl

#%%==========
# SECTION 1.2
#============

# fit equilibrium temperature change
# the function fmin find the couple of solutin coeff and T4x that minimize the error of the linear fit
coeff = np.zeros(len(model_list))
T4x = np.zeros(len(model_list))
for mod in model_list:
    m = model_list.index(mod)
    length = model_length[mod] #!!!
    def err(var):
        D_T = var[0] * (netImbalance[:length,m]) + var[1]
        return sum(( D_T - (Warming)[:length,m] )**2)
    [coeff[m],T4x[m]] = fmin(err,[-1.,3.],disp=False)


# deduce climate sensitivity
climsens = T4x / (5.35*np.log(4))

#============
# SECTION 2.1
#============
# Question 1 :  fit dynamical parameters
a_T1 = np.zeros(len(model_list))
tau_T1 = np.zeros(len(model_list))
tau_T2 = np.zeros(len(model_list))
for mod in model_list:
    m = model_list.index(mod)
    length = model_length[mod] #!!!
    time = np.arange(0,length)+0.5
    def err(var):
        D_T = T4x[m] * (1 - var[0]*np.exp(-time/var[1]) - (1-var[0])*np.exp(-time/var[2]))
        return sum(( D_T - (tas_quad-tas_ctrl)[:length,m] )**2)
    [a_T1[m],tau_T1[m],tau_T2[m]] = fmin(err,[0.5,2,200],disp=False)
a_T2 = 1-a_T1

# meaning of each parameters
#np.mean(tau_T1) ~3 years   => atmosphere box
#np.mean(tau_T2) ~150 years => ocean box
# The 2 time-scale simple climate models capture the behaviour of the coupled ocean-atmosphere response

# plot to evaluate the performance of the Simple Climate models at replicating the climate response of complex models
plt.figure()
for mod in model_list:
    m = model_list.index(mod)
    plt.subplot(5,5,m+1)
    timaxis=np.arange(250)
    SCM_tas = T4x[m]*(1 - a_T1[m]*np.exp(-timaxis/tau_T1[m]) - a_T2[m]*np.exp(-timaxis/tau_T2[m]))
    plt.plot((tas_quad-tas_ctrl)[:,m],marker='+',ls='none')    
    plt.plot(SCM_tas,lw=2)
    plt.title(mod)

# Question 2:  IRF for tas
def irf_tas(t,m):
    irf =  climsens[m] * ( a_T1[m]/tau_T1[m]*np.exp(-t/tau_T1[m]) + a_T2[m]/tau_T2[m] * np.exp(-t/tau_T2[m]))
    return irf
# Other solution
#irf_TAS = lambda t: climsens[m] * (a_T1[m]/tau_T1[m]*np.exp(-t/tau_T1[m]) + a_T2[m]/tau_T2[m]*np.exp(-t/tau_T2[m]))


# load the historical data on radiative forcing by IPCC
RF_comp = np.array([line for line in csv.reader(open("data/Historic_IPCC.1750-2011.RF.csv","r"))][1:],dtype=float)[:,:] #{W/m2}
RF_comp[:,-1] *= 0.7 # arbitrary climate efficiency for volcanoes
RF_lgd = [line for line in csv.reader(open("data/Historic_IPCC.1750-2011.RF.csv","r"))][0]
RF_all = np.sum(RF_comp,1)

# TESTING the principles of irf_tas
# using the built-in convolution function
#time = np.arange(len(RF_comp))
#m=1
#Tglo_model1 = np.convolve(RF_all,irf_tas(time,m))[:len(time)]

# simulate historical change in temperature using the irf_tas function
tax=np.arange(len(RF_all))+1750
D_TAS_all = np.zeros([len(RF_all),len(model_list)])
time = np.arange(len(RF_all))
for mod in model_list:
    m = model_list.index(mod)
    D_TAS_all[:,m] = np.convolve(RF_all,irf_tas(time,m))[:len(time)]
  
#============
# SECTION 2.2
#============ 
# load temperature reconstructions by NASA/GISS
GMT_giss = np.array([line for line in csv.reader(open('data/HistClim_GISS.1880-2014.gmt.csv','r'))],dtype=float)[:,0] #{degC}
tax_giss= np.arange(len(GMT_giss))+1880

# anomaly 1880-1899
D_TAS_giss = GMT_giss - np.mean(GMT_giss[0:19])


# Compare IRF and observations
plt.figure()
plt.plot(np.arange(1750,2012),np.zeros(2012-1750),'k-')
plt.plot(np.arange(1750,1750+len(D_TAS_all)),D_TAS_all,color='b')
plt.plot(np.arange(1880,1880+len(GMT_giss)),D_TAS_giss,color='k',marker='+',ls='none',lw=3)
plt.ylabel('global temperature change (K)')

# Attribute Changes in Global mean surface temperature (tas)

# FORCAGE CO2:
tax=np.arange(len(RF_all))+1750
D_TAS_CO2 = np.zeros([len(RF_all),len(model_list)])
time = np.arange(len(RF_all))
for mod in model_list:
    m = model_list.index(mod)
    D_TAS_CO2[:,m] = np.convolve(RF_comp[:,1],irf_tas(time,m))[:len(time)]


# FORCAGE VOLCAN:
tax=np.arange(len(RF_all))+1750
D_TAS_VOL = np.zeros([len(RF_all),len(model_list)])
time = np.arange(len(RF_all))
for mod in model_list:
    m = model_list.index(mod)
    D_TAS_VOL[:,m] = np.convolve(RF_comp[:,10],irf_tas(time,m))[:len(time)]


# FORCAGE ANTHROPIQUE:
RF_ANT = np.sum(RF_comp[:,:-2],1)
D_TAS_ANT = np.zeros([len(RF_ant),len(model_list)])
time = np.arange(len(RF_ant))
for mod in model_list:
    m = model_list.index(mod)
    D_TAS_ANT[:,m] = np.convolve(RF_ANT,irf_tas(time,m))[:len(time)]    


plt.figure()
plt.plot(tax,D_TAS_all[:,1],color='r',lw=1)
plt.plot(tax,D_TAS_CO2[:,1],color='o',lw=1)
plt.plot(tax,D_TAS_VOL[:,1],color='b',lw=1)
plt.plot(tax_giss,D_TAS_giss,color='k',marker='+',ls='none',lw=3)




